<?php {
  /*
   * $Id: dino_form.php,v 1.41 2003/04/01 11:06:32 nishina Exp $
   * Copyright (c) 2002-2003 Dino Co.,Ltd.
   */

  /**
   * formValidation
   * 
   * 
   * @package Dino_Form
   */

  /*
      ΥեִΥ

      date1 --> datey
      date2 --> datem
      date3 --> dated
      date4 --> dateh
      date5 --> datei
      date6 --> dates
  */
  require_once("form/tpl_form.inc.php");
  require_once("form/oohforms.inc.php");

  class Dino_form extends tpl_form {
    // tplvar_form_fields() ǡ̾ΥեǤget_element()롣
    var $elem_names = array();

    // ---------- public ----------
    var $use_js = true; // falseʤJavaScriptvalidatorĤʤ
    var $action = "";   // ǥեȤ$PHP_SELF
    var $after_js = "";  // finish()JS񤯡
    // $after_js="return check_submitting_once();" 2submitɻߤǤ롣
    var $before_js = ""; // finish()JS񤯡
    //------------------------------

    var $select_array = array();   // selectoptionꥹȤ¸array
    var $default_value = array();  // default value¸array
    var $select_array_year = array();
    var $select_array_month = array();
    var $select_array_day = array();
    var $select_array_hour = array();
    var $select_array_min = array();

    //᥽åɤλ   ǥեȤPOST
    var $use_post_method = true;

    // ---------- private ----------
    function init_select_array_date() {
      $cur_year  = date("Y");
      for($i = $cur_year-1; $i <= $cur_year+1; $i++){
	$this->select_array_year[] = array("label" => "$i" ,"value" => "$i");
      }
      for($i = 1; $i <= 12; $i++){
	$this->select_array_month[] = array("label" => "$i" ,"value" => "$i");
      }
      for($i = 1; $i <= 31; $i++){
	$this->select_array_day[] = array("label" => "$i" ,"value" => "$i");
      }
      for($i = 0; $i <= 23; $i++){
	$this->select_array_hour[] = array("label" => "$i" ,"value" => "$i");
      }
      for($i = 0; $i <= 59; $i++){
	$this->select_array_min[] = array("label" => "$i" ,"value" => "$i");
      }
    }

    function specify_method() {
      if($this->use_post_method){
	return "POST";
      }else {
	return "GET";
      }
    }

    function set_action($action) {
      $this->action = $action;
    }

    function init_select_array($name, $label = "ʲ򤷤Ƥ") {
      $this->select_array[$name] = array(array("label" => $label,
					       "value" => ""));

    }

    // obsolete
    function add_select_array($name, $label, $value="") {
      if (!is_array($label)) {
	if (!empty($label)) {
	  $this->select_array[$name][] = array("value" => $value,
					       "label" => $label);
	}
      } else {
	reset($label);
	while(list($k, $v) = each($label)) {
	  if (!empty($k)) {
	    $this->select_array[$name][] = array("value" => $k,
						 "label" => $v);
	  }
	}
      }
    }
    // obsolete
    function set_default_value($name, $value) {
      $this->default_value[$name] = $value;
    }

    // add_element()줿optionsɲä
    function append_options($name, $value, $label="") {
      if (!is_array($value)) {
	$this->form_data->elements[$name]["ob"]->options[] =
	   array("value" => $value, "label" => $label);
      } else {
	reset($value);
	while(list($k, $v) = each($value)) {
	  $this->form_data->elements[$name]["ob"]->options[] =
	     array("value" => $k, "label" => $v);
	}
      }
    }
    function change_default_value($name, $value) {
      if (is_object($this->form_data->elements[$name]["ob"])) {
	$this->form_data->elements[$name]["ob"]->value = $value;
      } else {
	// for backward compatibility
	$this->default_value[$name] = $value;
      }
    }

    // ɤʤ...
    function set_form_values($fv) {
      if (! is_object($this->form_data)) {
	$this->setup();
      }
      if (! is_array($fv)) {
	return false;
      }
      reset($fv);
      while (list($var, $value) = each($fv)) {
	if (ereg("^[a-zA-Z_][a-zA-Z0-9_]*$", $var)) {
	  if ($this->form_data->elements[$var]) {
	    global $$var;
	    $$var = $value;
	    $el_list[] = $var;
	  }
	}
      }
      if ($el_list) {
	$this->form_data->load_defaults($el_list);
      }
      return true;
    }
    function pre_disp() {
      global $form_name;
      if (! is_object($this->form_data)) {
	$this->setup();
      }
      if (($form_name == $this->classname) || $this->has_defaults) {
	$this->form_data->load_defaults();
      }
    }

    // եǤɽTemplate饹ѿ򥻥å
    function tplvar_form_fields($var_array = false) {
      $this->pre_disp(); // setup for 'special' hidden tag

      $form_prefix = $this->form_prefix;

      // set variable (formname)_start
      $jvs_name = "";
      if ($this->use_js) {
	$jvs_name = $form_prefix;
      }

      //$i = $form_prefix . "_start";
      //$tvars[$i] = $this->form_data->get_start($jvs_name, "", $this->action);

      // set variable (formname)_foo
      if (! is_array($var_array)) {
	// ʤȤ褦
      } else {
	reset($var_array);
	$this->elem_names = array(); // clear
	$regexp = "^" . $form_prefix . "[._]([^_]+(_[^_]+)*)(=|(__)([^=]+))?$";
	$form_started = false;
	while(list($k, $v) = each($var_array)) {
	  if (ereg($regexp, $v, $regs)) {
	    if (!$form_started) {
	      if (ereg("^(sys[-:])?start$", $regs[1])) {
		$form_started = true;
		$tvars[$v] = $this->form_data->get_start($jvs_name,
							 $this->specify_method(),
							 $this->action,
							 "",
							 $this->classname
							 );
	      }
	      continue;
	    } else {
	      if (ereg("^(sys[-:])?finish$", $regs[1])) {
		$tvars[$v] = $this->form_data->get_finish($this->after_js,
							  $this->before_js);
	      } else if ($regs[3]) {
		// special variables (for radio)
		$tvars[$v] = $this->form_data->get_element($regs[1], $regs[5]);
	      } else {
		// normal variables
		$tvars[$v] = $this->form_data->get_element($regs[1]);
	      }
	    }
	  }
	}
      }

      // set variable (formname)_finish
      //$i = $form_prefix . "_finish";
      //$tvars[$i] = $this->form_data->get_finish();

      return $tvars;
    }

    function get_start(){
      $this->pre_disp(); // setup for 'special' hidden tag

      $form_prefix = $this->form_prefix;

      // set variable (formname)_start
      $jvs_name = "";
      if ($this->use_js) {
	$jvs_name = $form_prefix;
      }

      //$i = $form_prefix . "_start";
      //$tvars[$i] = $this->form_data->get_start($jvs_name, "", $this->action);
      return $this->form_data->get_start($jvs_name,
					       $this->specify_method(),
					       $this->action,
					       "",
					       $this->classname
					       );

    }

    function get_finish(){
      return $this->form_data->get_finish($this->after_js,
					  $this->before_js);
    }

    function get_element($name,$value=false){
      // normal variables
      return $this->form_data->get_element($name,$value);
    }


    function get_radio_element($name,$num){
      // special variables (for radio)
      return $this->form_data->get_element($name,$num);
    }




    // DB ==> form
    //
    // overrideػߡ
    // ޸Ƥrowͤ顢default value롣
    
    // Ϣ󤫤
    function set_default_values_from_assoc(&$values) {
      $fv = array();
      $this->values2fv(&$values, &$fv);
      $fv["form_name"] = $this->classname;
      $this->set_default_values(&$fv);
    }

    function set_default_values_from_db(&$db) {
      $values = $db->Record;
      $this->set_default_values_from_assoc($values);
    }

    function set_default_values_from_peardb(&$rs) {
      $array = array();
      if(!$values = $rs->fetchRow(DB_FETCHMODE_ASSOC)) {
	return false;
      }
      $this->set_default_values_from_assoc($values);
    }
    
    // DB ==> form
    //
    // Τޤ
    function values2fv_all(&$values, &$fv, $suffix = "") {
      reset($values);
      while (list($k, $v) = each($values)) {
	if (ereg("^[0-9]+$", $k)) {
	  // ե̾¸ߤʤ
	} else {
	  //echo $k, $suffix, " = ", $v, "<br>";
	  $fv[$k.$suffix] = $v;
	}
      }
    }
    /**
     *  DB ==> form
     * 
     *  (餯)DBäƤͤ set_default_values() Ϥ褦ˤ
     *  (ٹɬפʤȤ⤢Ϥ/selectʸǺٹƤɤ)
     * 
     *  overrideƻȤ
     *  $valuesϽ񤭴ᡣ
     *  $fvϾOK
     *
     * @access public
     */
    function values2fv(&$values, &$fv, $suffix = "") {
      $this->values2fv_all($values, $fv, $suffix); // $fvϾ񤭤
    }

    /**
     * form ==> DB
     *
     *
     * @access public
     */
    function process_input() {
      $fv = $this->get_default_values();
      $this->fv2values($fv, $this->values);

      unset($this->values["form_name"]);
      unset($this->values["submit"]);

      return $this->values;
    }
    /**
     *  form ==> DB
     *
     *  queryȤѿ̾=ͤ褿ˡ
     *  $values["ѿ̾"] = ""
     *  ѿ̾ѿ̾1, ѿ̾2Ĥʤɤξ
     *  $values[1]["ѿ̾"] = ""ʤɤȤ롣
     *
     * @access public
     */
    function fv2values_all(&$fv, &$values, $suffix_flag = false) {
      reset($fv);
      if ($suffix_flag) {
	while (list($k, $v) = each($fv)) {
	  if (ereg("^(.*[^0-9])([0-9]+)$", $k, $regs)) {
	    $values[$regs[2]][$regs[1]] = $v;
	  } else {
	    $values[$k] = $v;
	  }
	}
      } else {
	while (list($k, $v) = each($fv)) {
	  $values[$k] = $v;
	}
      }
    }

    /**
     *  form ==> DB
     * 
     *  formϤäƤ͡get_default_values()
     *  äƤͤٹ $values ˵ͤ롣
     *  ͤϤΤޤsql::update()ʤɤϤϤ
     * 
     *  overrideƻȤ
     *  $fvϽ񤭴ᡣ
     *  $valuesϾOK
     *
     * @access public
     */
    function fv2values(&$fv, &$values) {
      $this->fv2values_all($fv, $values); // $fvϾ񤭤
    }

    function values2fv_timestamp(&$values, &$fv, $form_name, $suffix = "") {
      if (ereg("^(([0-9]+)-([0-9][0-9])-([0-9][0-9]))? *(([0-9][0-9]):([0-9][0-9]):([0-9][0-9]))?",
	       $values[$form_name], $regs)) {
	if ($regs[1]) {
	  $fv[$form_name."-y".$suffix] = $regs[2];
	  $fv[$form_name."-m".$suffix] = $regs[3];
	  $fv[$form_name."-d".$suffix] = $regs[4];
	}
	if ($regs[5]) {
	  $fv[$form_name."-h".$suffix] = $regs[6];
	  $fv[$form_name."-i".$suffix] = $regs[7];
	  $fv[$form_name."-s".$suffix] = $regs[8];
	}
      }
    }
    function values2fv_date(&$values, &$fv, $form_name, $suffix = "") {
      $this->values2fv_timestamp(&$values, &$fv, &$form_name, &$suffix);
    }
    function values2fv_time(&$values, &$fv, $form_name, $suffix = "") {
      $this->values2fv_timestamp(&$values, &$fv, &$form_name, &$suffix);
    }

    function fv2values_timestamp(&$fv, &$values, $form_name, $suffix = "") {
      $dateform_suffix = array(1=>"y", 2=>"m", 3=>"d", 4=>"h", 5=>"i", 6=>"s");

      $array = array(1=>1970, 2=>1, 3=>1, 4=>0, 5=>0, 6=>0);

      for ($i = 1; $i <= 6; $i++) {
	$key = $form_name."-".$dateform_suffix[$i].$suffix;
	if (isset($fv[$key])) {
	  if ($from == 0) $from = $i;
	  $to = $i;
	  $array[$i] = $fv[$key];
	}
      }
      $time = mktime($array[4], $array[5], $array[6],
		     $array[2], $array[3], $array[1]);
      if ($from >= 4 && $to <= 6) {
	// time
	$value = date("H:i:s", $time);
      } else {
	// timestamp/date
	$value = date("Y-m-d H:i:s+09", $time);
      }
      if ((string)$suffix == "") {
	$values[$form_name] = $value;
      } else {
	$values[$suffix][$form_name] = $value;
      }
    }
    function fv2values_date(&$fv, &$values, $form_name, $suffix = "") {
      $this->fv2values_timestamp(&$fv, &$values, &$form_name, &$suffix);
    }
    function fv2values_time(&$fv, &$values, $form_name, $suffix = "") {
      $this->fv2values_timestamp(&$fv, &$values, &$form_name, &$suffix);
    }


    function values2dateform(&$values, &$fv, $form_name, $from, $to, $suffix = "") {
      $dateform_suffix = array(1=>"y", 2=>"m", 3=>"d", 4=>"h", 5=>"i", 6=>"s");
	    
      if ($from < 1) { $from = 1; }
      if ($to   > 6) { $to   = 6; }

      if (ereg("^([0-9]+)-([0-9][0-9])-([0-9][0-9]) ([0-9][0-9]):([0-9][0-9]):([0-9][0-9])",
	       $values[$form_name], $array)) {
	for ($i = $from; $i <= $to; $i++) {
	  $fv[$form_name.$dateform_suffix[$i].$suffix] = $array[$i];
	}
      }
    }

    function dateform2values(&$fv, &$values, $form_name, $from, $to, $suffix = "") {
      $dateform_suffix = array(1=>"y", 2=>"m", 3=>"d", 4=>"h", 5=>"i", 6=>"s");

      $array = array(1=>"1970", 2=>"1", 3=>"1", 4=>"0", 5=>"0", 6=>"0");

      for ($i = $from; $i <= $to; $i++) {
	$key = $form_name.$dateform_suffix[$i].$suffix;
	if (isset($fv[$key])) {
	  $array[$i] = $fv[$key];
	}
      }
      $time = mktime($array[4], $array[5], $array[6],
		     $array[2], $array[3], $array[1]);
      if ($from >= 4 && $to <= 6) {
	// time
	$value = date("H:i:s", $time);
      } else {
	// timestamp/date
	$value = date("Y-m-d H:i:s+09", $time);
      }
      if ((string)$suffix == "") {
	$values[$form_name] = $value;
      } else {
	$values[$suffix][$form_name] = $value;
      }

      return true;
    }

    function setup_numerical_select_form($form_name, $def, $from, $to, $step=1) {
      $elemarray = array(
			 "name"    => $form_name,
			 "type"    => "select",
			 "options" => array(),
			 "size"    => 1,
			 "value"   => $def,
			 );
      for ($i = $from; $i <= $to; $i+=$step){
	$elemarray["options"][] = array("value"=>$i,"label"=>$i);
	//$elemarray["options"][] = $i;
      }
      $this->form_data->add_element($elemarray);
    }

    function setup_dateform($form_name, $from, $to, $suffix = "") {
      if (sizeof($this->select_array_year) == 0) {
	$this->init_select_array_date();
      }

      $dateform_suffix = array(1=>"y", 2=>"m", 3=>"d", 4=>"h", 5=>"i", 6=>"s");

      $cur_year  = date("Y");
      $cur_month = date("m");
      $cur_day   = date("d");
	  
      if ($from < 1) { $from = 1; }
      if ($to   > 6) { $to   = 6; }

      for ($i = $from; $i <= $to; $i++) {
	$array[$i] = $fv[$form_name.$dateform_suffix[$i]];
      }


      if ($from <= 1 && $to >= 1) {
	$this->form_data->add_element(array(
					    "name"    => $form_name."y".$suffix,
					    "type"    => "text",

					    "value"   => $cur_year,

					    "size"    => 5,
					    "maxlength"    => 4,
					    "valid_regex"  => "^[0-9][0-9][0-9][0-9]$",
					    "valid_e" => "ǯȾѿ4ϤƤ"
					    ));
      }

      if ($from <= 2 && $to >= 2) {
	for($i=1; $i<=12; $i++){
	  $m[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => $form_name."m".$suffix,
					    "type"    => "select",
					    "options" => $this->select_array_month,
					    "size"    => 1,
					    "value"   => $cur_month
					    ));
      }


      if ($from <= 3 && $to >= 3) {
	for($i=1; $i<=31; $i++){
	  $d[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => $form_name."d".$suffix,
					    "type"    => "select",
					    "options" => $this->select_array_day,
					    "size"    => 1,
					    "value"   => $cur_day
					    ));
      }
      if ($from <= 4 && $to >= 4) {
	for($i=0; $i<24; $i++){
	  $h[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => $form_name."h".$suffix,
					    "type"    => "select",
					    "options" => $this->select_array_hour,
					    "size"    => 1,
					    "value"   => 0
					    ));
      }
      if ($from <= 5 && $to >= 5) {
	for($i=0; $i<60; $i++){
	  $a5[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => $form_name."i".$suffix,
					    "type"    => "select",
					    "options" => $this->select_array_min,
					    "size"    => 1,
					    "value"   => 0
					    ));
      }
      if ($from <= 6 && $to >= 6) {
	for($i=0; $i<60; $i++){
	  $a6[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => $form_name."s".$suffix,
					    "type"    => "select",
					    "options" => $this->select_array_min,
					    "size"    => 1,
					    "value"   => 0
					    ));
      }
    }

    function setup_datespan($from, $to) {
      if ($from <= 1 && $to >= 1) {
	for($i=2001; $i<=2002; $i++){
	  $y[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => "from1",
					    "type"    => "select",
					    "options" => $y,
					    "size"    => 1,
					    "value"   => 2001
					    ));
	$this->form_data->add_element(array(
					    "name"    => "to1",
					    "type"    => "select",
					    "options" => $y,
					    "size"    => 1,
					    "value"   => 2001
					    ));
      }

      if ($from <= 2 && $to >= 2) {
	for($i=1; $i<=12; $i++){
	  $m[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => "from2",
					    "type"    => "select",
					    "options" => $m,
					    "size"    => 1,
					    "value"   => 1
					    ));
	$this->form_data->add_element(array(
					    "name"    => "to2",
					    "type"    => "select",
					    "options" => $m,
					    "size"    => 1,
					    "value"   => 12
					    ));
      }


      if ($from <= 3 && $to >= 3) {
	for($i=1; $i<=31; $i++){
	  $d[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => "from3",
					    "type"    => "select",
					    "options" => $d,
					    "size"    => 1,
					    "value"   => 1
					    ));
	$this->form_data->add_element(array(
					    "name"    => "to3",
					    "type"    => "select",
					    "options" => $d,
					    "size"    => 1,
					    "value"   => 31
					    ));

      }
      if ($from <= 4 && $to >= 4) {
	for($i=0; $i<24; $i++){
	  $h[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => "from4",
					    "type"    => "select",
					    "options" => $h,
					    "size"    => 1,
					    "value"   => 0
					    ));

	$this->form_data->add_element(array(
					    "name"    => "to4",
					    "type"    => "select",
					    "options" => $h,
					    "size"    => 1,
					    "value"   => 0
					    ));

      }
      if ($from <= 5 && $to >= 5) {
	for($i=0; $i<60; $i++){
	  $a5[] = array("label" => "$i" ,"value" => "$i");
	}
	$this->form_data->add_element(array(
					    "name"    => "from5",
					    "type"    => "select",
					    "options" => $a5,
					    "size"    => 1,
					    "value"   => 0
					    ));

	$this->form_data->add_element(array(
					    "name"    => "to5",
					    "type"    => "select",
					    "options" => $a5,
					    "size"    => 1,
					    "value"   => 0
					    ));
      }
    }

  }

}?>
